@@ -63,8 +63,8 @@
 
 use std::error::Error;
 use std::fmt;
+use std::ptr::NonNull;
 use libdeflate_sys::{libdeflate_decompressor,
-                            libdeflate_alloc_decompressor,
                             libdeflate_free_decompressor,
                             libdeflate_gzip_decompress,
                             libdeflate_zlib_decompress,
@@ -74,7 +74,6 @@ use libdeflate_sys::{libdeflate_decompressor,
                             libdeflate_result_LIBDEFLATE_BAD_DATA,
                             libdeflate_result_LIBDEFLATE_INSUFFICIENT_SPACE,
                             libdeflate_compressor,
-                            libdeflate_alloc_compressor,
                             libdeflate_deflate_compress_bound,
                             libdeflate_deflate_compress,
                             libdeflate_zlib_compress_bound,
@@ -87,16 +86,25 @@ use libdeflate_sys::{libdeflate_decompressor,
 
 #[cfg(feature = "use_rust_alloc")]
 mod malloc_wrapper;
-#[cfg(feature = "use_rust_alloc")]
-use malloc_wrapper::init_allocator;
 
-#[cfg(not(feature = "use_rust_alloc"))]
-fn init_allocator() {}
+unsafe fn alloc_compressor(compression_level: std::os::raw::c_int) -> *mut libdeflate_compressor {
+    #[cfg(feature = "use_rust_alloc")]
+    { libdeflate_sys::libdeflate_alloc_compressor_ex(compression_level, &malloc_wrapper::OPTIONS) }
+    #[cfg(not(feature = "use_rust_alloc"))]
+    { libdeflate_sys::libdeflate_alloc_compressor(compression_level) }
+}
+
+unsafe fn alloc_decompressor() -> *mut libdeflate_decompressor {
+    #[cfg(feature = "use_rust_alloc")]
+    { libdeflate_sys::libdeflate_alloc_decompressor_ex(&malloc_wrapper::OPTIONS) }
+    #[cfg(not(feature = "use_rust_alloc"))]
+    { libdeflate_sys::libdeflate_alloc_decompressor() }
+}
 
 /// A `libdeflate` decompressor that can inflate DEFLATE, zlib, or
 /// gzip data.
 pub struct Decompressor {
-    p: *mut libdeflate_decompressor,
+    p: NonNull<libdeflate_decompressor>,
 }
 unsafe impl Send for Decompressor {}
 
@@ -129,15 +137,20 @@ impl Error for DecompressionError {}
 /// A result returned by decompression methods
 type DecompressionResult<T> = std::result::Result<T, DecompressionError>;
 
+impl Default for Decompressor {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 #[allow(non_upper_case_globals)]
 impl Decompressor {
 
     /// Returns a newly constructed instance of a `Decompressor`.
     pub fn new() -> Decompressor {
         unsafe {
-            init_allocator();
-            let ptr = libdeflate_alloc_decompressor();
-            if !ptr.is_null() {
+            let ptr = alloc_decompressor();
+            if let Some(ptr) = NonNull::new(ptr) {
                 Decompressor{ p: ptr }
             } else {
                 panic!("libdeflate_alloc_decompressor returned NULL: out of memory");
@@ -155,12 +168,11 @@ impl Decompressor {
                            gz_data: &[u8],
                            out: &mut [u8]) -> DecompressionResult<usize> {
         unsafe {
-            init_allocator();
             let mut out_nbytes = 0;
             let in_ptr = gz_data.as_ptr() as *const std::ffi::c_void;
             let out_ptr = out.as_mut_ptr() as *mut std::ffi::c_void;
             let ret: libdeflate_result =
-                libdeflate_gzip_decompress(self.p,
+                libdeflate_gzip_decompress(self.p.as_ptr(),
                                            in_ptr,
                                            gz_data.len(),
                                            out_ptr,
@@ -193,12 +205,11 @@ impl Decompressor {
                            zlib_data: &[u8],
                            out: &mut [u8]) -> DecompressionResult<usize> {
         unsafe {
-            init_allocator();
             let mut out_nbytes = 0;
             let in_ptr = zlib_data.as_ptr() as *const std::ffi::c_void;
             let out_ptr = out.as_mut_ptr() as *mut std::ffi::c_void;
             let ret: libdeflate_result =
-                libdeflate_zlib_decompress(self.p,
+                libdeflate_zlib_decompress(self.p.as_ptr(),
                                            in_ptr,
                                            zlib_data.len(),
                                            out_ptr,
@@ -232,12 +243,11 @@ impl Decompressor {
                               deflate_data: &[u8],
                               out: &mut [u8]) -> DecompressionResult<usize> {
         unsafe {
-            init_allocator();
             let mut out_nbytes = 0;
             let in_ptr = deflate_data.as_ptr() as *const std::ffi::c_void;
             let out_ptr = out.as_mut_ptr() as *mut std::ffi::c_void;
             let ret: libdeflate_result =
-                libdeflate_deflate_decompress(self.p,
+                libdeflate_deflate_decompress(self.p.as_ptr(),
                                               in_ptr,
                                               deflate_data.len(),
                                               out_ptr,
@@ -265,7 +275,7 @@ impl Decompressor {
 impl Drop for Decompressor {
     fn drop(&mut self) {
         unsafe {
-            libdeflate_free_decompressor(self.p);
+            libdeflate_free_decompressor(self.p.as_ptr());
         }
     }
 }
@@ -302,7 +312,7 @@ impl CompressionLvl {
     ///
     /// Valid compression levels for libdeflate, at time of writing,
     /// are 1-12.
-    pub fn new(level: i32) -> CompressionLevelResult {
+    pub const fn new(level: i32) -> CompressionLevelResult {
         if MIN_COMPRESSION_LVL <= level && level <= MAX_COMPRESSION_LVL {
             Ok(CompressionLvl(level))
         } else {
@@ -312,20 +322,20 @@ impl CompressionLvl {
 
     /// Returns the fastest compression level. This compression level
     /// offers the highest performance but lowest compression ratio.
-    pub fn fastest() -> CompressionLvl {
+    pub const fn fastest() -> CompressionLvl {
         CompressionLvl(MIN_COMPRESSION_LVL)
     }
 
     /// Returns the best compression level, in terms of compression
     /// ratio. This compression level offers the best compression
     /// ratio but lowest performance.
-    pub fn best() -> CompressionLvl {
+    pub const fn best() -> CompressionLvl {
         CompressionLvl(MAX_COMPRESSION_LVL)
     }
 
     /// Returns an iterator that emits all compression levels
     /// supported by `libdeflate` in ascending order.
-    pub fn iter() -> CompressionLvlIter {
+    pub const fn iter() -> CompressionLvlIter {
         CompressionLvlIter(MIN_COMPRESSION_LVL)
     }
 }
@@ -391,10 +401,16 @@ type CompressionResult<T> = std::result::Result<T, CompressionError>;
 /// A `libdeflate` compressor that can compress arbitrary data into
 /// DEFLATE, zlib, or gzip formats.
 pub struct Compressor {
-    p: *mut libdeflate_compressor,
+    p: NonNull<libdeflate_compressor>,
 }
 unsafe impl Send for Compressor {}
 
+impl Default for Compressor {
+    fn default() -> Self {
+        Self::new(CompressionLvl::default())
+    }
+}
+
 impl Compressor {
 
     /// Returns a newly constructed `Compressor` that compresses data
@@ -402,9 +418,8 @@ impl Compressor {
     /// [`CompressionLvl`](struct.CompressionLvl.html)
     pub fn new(lvl: CompressionLvl) -> Compressor {
         unsafe {
-            init_allocator();
-            let ptr = libdeflate_alloc_compressor(lvl.0);
-            if !ptr.is_null() {
+            let ptr = alloc_compressor(lvl.0 as _);
+            if let Some(ptr) = NonNull::new(ptr) {
                 Compressor{ p: ptr }
             } else {
                 panic!("libdeflate_alloc_compressor returned NULL: out of memory");
@@ -419,7 +434,7 @@ impl Compressor {
     /// compressed), format overhead, etc.
     pub fn deflate_compress_bound(&mut self, n_bytes: usize) -> usize {
         unsafe {
-            libdeflate_deflate_compress_bound(self.p, n_bytes)
+            libdeflate_deflate_compress_bound(self.p.as_ptr(), n_bytes)
         }
     }
 
@@ -431,11 +446,10 @@ impl Compressor {
                             in_raw_data: &[u8],
                             out_deflate_data: &mut [u8]) -> CompressionResult<usize> {
         unsafe {
-            init_allocator();
             let in_ptr = in_raw_data.as_ptr() as *const std::ffi::c_void;
             let out_ptr = out_deflate_data.as_mut_ptr() as *mut std::ffi::c_void;
 
-            let sz = libdeflate_deflate_compress(self.p,
+            let sz = libdeflate_deflate_compress(self.p.as_ptr(),
                                                  in_ptr,
                                                  in_raw_data.len(),
                                                  out_ptr,
@@ -456,7 +470,7 @@ impl Compressor {
     /// compressed), format overhead, etc.
     pub fn zlib_compress_bound(&mut self, n_bytes: usize) -> usize {
         unsafe {
-            libdeflate_zlib_compress_bound(self.p, n_bytes)
+            libdeflate_zlib_compress_bound(self.p.as_ptr(), n_bytes)
         }
     }
 
@@ -468,11 +482,10 @@ impl Compressor {
                          in_raw_data: &[u8],
                          out_zlib_data: &mut [u8]) -> CompressionResult<usize> {
         unsafe {
-            init_allocator();
             let in_ptr = in_raw_data.as_ptr() as *const std::ffi::c_void;
             let out_ptr = out_zlib_data.as_mut_ptr() as *mut std::ffi::c_void;
 
-            let sz = libdeflate_zlib_compress(self.p,
+            let sz = libdeflate_zlib_compress(self.p.as_ptr(),
                                               in_ptr,
                                               in_raw_data.len(),
                                               out_ptr,
@@ -493,7 +506,7 @@ impl Compressor {
     /// compressed), format overhead, etc.
     pub fn gzip_compress_bound(&mut self, n_bytes: usize) -> usize {
         unsafe {
-            libdeflate_gzip_compress_bound(self.p, n_bytes)
+            libdeflate_gzip_compress_bound(self.p.as_ptr(), n_bytes)
         }
     }
 
@@ -505,11 +518,10 @@ impl Compressor {
                          in_raw_data: &[u8],
                          out_gzip_data: &mut [u8]) -> CompressionResult<usize> {
         unsafe {
-            init_allocator();
             let in_ptr = in_raw_data.as_ptr() as *const std::ffi::c_void;
             let out_ptr = out_gzip_data.as_mut_ptr() as *mut std::ffi::c_void;
 
-            let sz = libdeflate_gzip_compress(self.p,
+            let sz = libdeflate_gzip_compress(self.p.as_ptr(),
                                               in_ptr,
                                               in_raw_data.len(),
                                               out_ptr,
@@ -527,7 +539,7 @@ impl Compressor {
 impl Drop for Compressor {
     fn drop(&mut self) {
         unsafe {
-            libdeflate_free_compressor(self.p);
+            libdeflate_free_compressor(self.p.as_ptr());
         }
     }
 }
@@ -538,9 +550,15 @@ pub struct Crc {
     val: u32,
 }
 
+impl Default for Crc {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Crc {
     /// Returns a new `Crc` instance
-    pub fn new() -> Crc {
+    pub const fn new() -> Crc {
         Crc { val: 0 }
     }
 
@@ -554,7 +572,7 @@ impl Crc {
     }
 
     /// Returns the current CRC32 checksum
-    pub fn sum(&self) -> u32 {
+    pub const fn sum(&self) -> u32 {
         self.val
     }
 }
@@ -576,9 +594,15 @@ pub struct Adler32 {
     val: u32,
 }
 
+impl Default for Adler32 {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl Adler32 {
     /// Returns a new `Adler32` instance (with initial adler32 value 1, which is default for adler32)
-    pub fn new() -> Adler32 {
+    pub const fn new() -> Adler32 {
         Adler32 { val: 1 }
     }
     /// Update the Adler32 with the bytes in `data`
@@ -591,7 +615,7 @@ impl Adler32 {
             }
     }
     /// Returns the current Adler32 checksum
-    pub fn sum(&self) -> u32 {
+    pub const fn sum(&self) -> u32 {
         self.val
     }
 }
